<?php
/**
 * Plugin Name: Bizuno API
 * Plugin URI: https://www.phreesoft.com
 * Description: The Bizuno-WordPress plugin allow you to synchronize data between any Bizuno host and your WooCommerce cart. Uploads inventory and order status to the cart, download orders to your Bizuno business.
 * Version: 3.2.7
 * Author: David Premo, PhreeSoft, Inc.
 * WC requires at least: 4.4
 * WC tested up to: 5.2
 */

if (!defined('ABSPATH')) { die('No script kiddies please!'); }

if (!defined('BIZUNO_API_PATH')) { define('BIZUNO_API_PATH', plugin_dir_path( __FILE__ )); }

/**
 * Check if WooCommerce is active
 **/
if ( !in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option( 'active_plugins' )))) { // log error / display error
    deactivate_plugins(plugin_basename(__FILE__)); // Deactivate ourselves
    wp_die( __('Did not find WooCommerce, please activate this plugin before you activate the Bizuno API.', 'bizuno_api'), __('Need WooCommerce 3.3.X', 'bizuno_api'), ['back_link' => true]);
}

add_action( 'wp_ajax_bizuno_api', 'bizuno_api' );
add_action( 'wp_ajax_nopriv_bizuno_api', 'bizuno_api' );
function bizuno_api() {
    require_once('bizuno-lib.php');
    $ctl = new \bizunoapi\bizunoAPI();
    $ctl->processRcv();
}

/**
 * Adds download button, two version pre 3.3 and post 3.3
 */
if (is_admin()):
if (!function_exists('bizuno_api_get_order_meta')):
    function bizuno_api_get_order_meta($order, $key='', $single=true, $context='edit') {
        if (defined( 'WC_VERSION' ) && WC_VERSION && version_compare( WC_VERSION, '3.0', '>=' )) { // WooCommerce > 3.0
            $value   = $order->get_meta($key, $single, $context);
        } else {  // have the $order->get_id() check here just in case the WC_VERSION isn't defined correctly
            $order_id= is_callable([$order, 'get_id'] ) ? $order->get_id() : $order->id;
            $value   = get_post_meta($order_id, $key, $single);
        }
        return $value;
    }
endif;
function bizuno_woo_early_version() {
    if (!function_exists('get_plugin_data')) {
        require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    }
    $info = get_plugin_data(WP_PLUGIN_DIR.'/woocommerce/woocommerce.php');
    return version_compare($info['Version'], '3.3', "<" ) ? true : false;
}
if (bizuno_woo_early_version()) {
    add_filter('woocommerce_admin_order_actions', 'bizuno_api_order_download_action', 10, 2);
    function bizuno_api_order_download_action($actions, $order) {
        $active = get_option('bizuno_api_active', false );
        $exported = bizuno_api_get_order_meta( $order, 'bizuno_order_exported' );
        if ( $exported == '0' && $active == 1 ) {
            $actions['download'] = [
                'url'    => admin_url( 'admin-ajax.php?action=bizuno_api_order_download&biz_ajax=1&biz_order_id='.$order->id ),
                'name'    => __( 'Download', 'bizuno_api' ),
                'action'=> "download"];
        }
        return $actions;
    }
} else { // WooCommerce Version >= 3.3
    // Add column to order summary page
    add_filter( 'manage_edit-shop_order_columns', 'bizuno_api_order_column_header', 20 );
    function bizuno_api_order_column_header( $columns ) {
        $new_columns = [];
        foreach ( $columns as $column_name => $column_info ) {
            if ( 'order_date' === $column_name ) { $new_columns['bizuno_download'] = __( 'Exported', 'bizuno_api' ); }
            $new_columns[$column_name] = $column_info;
        }
        return $new_columns;
    }
    add_action( 'manage_shop_order_posts_custom_column', 'bizuno_api_order_column_content' );
    function bizuno_api_order_column_content($column) {
        global $post;
        if ( 'bizuno_download' === $column ) {
            $order    = wc_get_order( $post->ID );
            $exported = bizuno_api_get_order_meta( $order, 'bizuno_order_exported' );
            if (!$exported) {
                $tip = '';
                echo '<button type="button" class="order-status status-processing tips" data-tip="'.$tip.'">'.__( 'No', 'bizuno_api' ).'</button>';
            } else { echo '&nbsp;'; }
        }
    }
    // Add download button to Preview pop up
    add_filter( 'woocommerce_admin_order_preview_get_order_details', 'bizuno_api_order_preview_filter', 10, 2);
    function bizuno_api_order_preview_filter( $data, $order ) {
        $data['bizuno_order_exported'] = $order->get_meta('bizuno_order_exported', true, 'edit') ? 'none' : 'block';
        return $data;
    }
    add_action( 'woocommerce_admin_order_preview_end', 'bizuno_api_order_preview_action');
    function bizuno_api_order_preview_action( $order) {
//        $url = add_query_arg(['bizuno_api_order_download'=>'1'], esc_url(admin_url( 'edit.php?post_type=shop_order')) );
        $url = admin_url( 'admin-ajax.php?action=bizuno_api_order_download' );
        echo '<span style="display:{{ data.bizuno_order_exported }}"><a class="button button-primary button-large" onClick="window.location = \''.$url.'&biz_order_id={{ data.data.id }}\';">'.__( 'Export order to Bizuno', 'bizuno_api' ).'</a></span>'."\n";
    }
    // add download button to order edit page
    add_filter( 'woocommerce_order_actions', 'bizuno_api_add_order_meta_box_filter' );
    function bizuno_api_add_order_meta_box_filter( $actions ) {
        global $post;
        if (get_post_meta( $post->ID, 'bizuno_order_exported', true ) ) { return $actions; }
        $actions['bizuno_export_action'] = __('Export order to Bizuno', 'bizuno_api');
        return $actions;
    }
    add_action( 'woocommerce_order_action_bizuno_export_action', 'bizuno_api_process_order_meta_box_action' );
    function bizuno_api_process_order_meta_box_action( $order ) {
        bizuno_api_manual_download($order->id);
    }
}
/**
 * Process for manual download of an order
 */
add_action('wp_ajax_bizuno_api_order_download','bizuno_api_manual_download', 10);
function bizuno_api_manual_download($order_id = 0) {
    $active = get_option('bizuno_api_active', false);
    if ($active == '') { return; }
    if (!$order_id) { $order_id = (int)$_GET['biz_order_id']; }
    $result = bizuno_api_order_download($order_id);
    wp_redirect( esc_url(admin_url( 'edit.php?post_type=shop_order') ) );
    exit;
}
endif;

/**
 * Adds Bizuno meta to order (last call before completed)
 * May also download order if set to do so
 */
add_action( 'woocommerce_thankyou','bizuno_api_post_payment', 10, 1);
// add_action('woocommerce_payment_complete', 'bizuno_api_post_payment', 10, 1); // This hook doesn't capture the converge meta data additions, tx_id and hint, etc
function bizuno_api_post_payment($order_id) {
    // set the exported meta to 'not downloaded'
    if (!in_array('bizuno_order_exported', get_post_custom_keys($order_id))) { update_post_meta($order_id, 'bizuno_order_exported', 0); }
    // auto download, don't show any messages to customer
    if ('yes' == strtolower(get_option('bizuno_api_autodownload', false))) {
        bizuno_api_order_download($order_id); // call return to bit bucket as as all messsages are suppressed
        $order = new WC_Order($order_id);
        $order->update_status('completed');
    }
}

/**
 * Calls the API method processSend to send to Bizuno
 * @global object $wpdb
 * @param integer $order_id
 */
function bizuno_api_order_download($order_id)
{
    global $wpdb; // Database
    $GLOBALS['dbBizuno'] = ['type'=>'mysql','host'=>DB_HOST,'name'=>DB_NAME,'user'=>DB_USER,'pass'=>DB_PASSWORD,'prefix'=>$wpdb->prefix];
    require_once(dirname(__FILE__).'/bizuno-lib.php');
    $ctl   = new \bizunoapi\bizunoAPI();
    $ctl->msgDebug("\nPassed order_id = $order_id");
    $arg   = false;
    $data  = apply_filters('bizuno_api_data', $arg, $order_id);
    $ctl->processSend('downloadOrder', $order_id, $data);
}

// Add notices to the queue to display after page redirect
function bizuno_api_add_notice( $notice = "", $type = "warning", $dismissible = true ) {
    $notices = get_option( "bizuno_flash_notices", [] );
    $dismissible_text = ($dismissible) ? "is-dismissible" : "";
    array_push( $notices, ["notice"=>$notice, "type"=>$type, "dismissible"=>$dismissible_text] );
    update_option("bizuno_flash_notices", $notices );
}
function bizuno_api_display_notices() {
    $notices = get_option( "bizuno_flash_notices", [] );
    foreach ( $notices as $notice ) {
        printf('<div class="notice notice-%1$s %2$s"><p>%3$s</p></div>', $notice['type'], $notice['dismissible'], $notice['notice'] );
    }
    if (!empty( $notices)) { delete_option( "bizuno_flash_notices", [] ); }
}
add_action( 'admin_notices', 'bizuno_api_display_notices', 12 );

/**
 * set default settings fields and post meta
 */
register_activation_hook(__FILE__ , 'bizuno_api_activate' );
function bizuno_api_activate() {
    global $wpdb;
    update_option('bizuno_api_active', true); //activate
    update_option('bizuno_api_url', 'https://www.bizuno.com', false); //set default url
    // set all existing orders to downloaded to hide Download button
    $orders = $wpdb->get_results( "SELECT `ID` FROM `{$wpdb->prefix}posts` WHERE `post_type` LIKE 'shop_order'", ARRAY_A);
     foreach ($orders as $order) { update_post_meta($order['ID'], 'bizuno_order_exported', 1); }
}

/**
 * This de-activates the plugin and the ability to upload and download.
 */
register_deactivation_hook(__FILE__ , 'bizuno_api_deactivate' );
function bizuno_api_deactivate() {
    update_option('bizuno_api_active', false);
}

/**
 * This removes all information related to this plugin
 */
register_uninstall_hook(__FILE__, 'bizuno_api_uninstall');
function bizuno_api_uninstall() {
    global $wpdb;
    delete_option('bizuno_api_user');
    delete_option('bizuno_api_pw');
    delete_option('bizuno_api_url');
    delete_option('bizuno_api_prefix');
    delete_option('bizuno_api_prefix_cust');
    delete_option('bizuno_api_autodownload');
    delete_option('bizuno_api_active');
    //multi site
    delete_site_option('bizuno_api_user');
    delete_site_option('bizuno_api_pw');
    delete_site_option('bizuno_api_url');
    delete_site_option('bizuno_api_prefix');
    delete_site_option('bizuno_api_prefix_cust');
    delete_site_option('bizuno_api_autodownload');
    delete_site_option('bizuno_api_active');
    //add order fields
    $orders = $wpdb->get_results( "SELECT `ID` FROM `{$wpdb->prefix}posts` WHERE `post_type` LIKE 'shop_order'", ARRAY_A);
    foreach ($orders as $order) { delete_post_meta($order['ID'], 'bizuno_order_exported'); }
}

/**
 * Add settings to the specific section we created before
 */
add_filter('woocommerce_get_sections_advanced', 'bizuno_api_add_section');
function bizuno_api_add_section( $sections ) {
    $sections['bizuno_api'] = __( 'Bizuno API', 'text-domain' );
    return $sections;
}

add_filter('woocommerce_get_settings_advanced', 'bizuno_api_all_setting', 10, 2 );
function bizuno_api_all_setting( $settings, $current_section ) {
    if ( $current_section <> 'bizuno_api' ) { return $settings; }
    unset($settings);
    $settings[] = [
        'name'=>__( 'Bizuno API', 'text-domain' ),
        'type'=>'title',
        'desc'=>__( 'The following options are used to configure the interface to Bizuno Accounting', 'text-domain' ),
        'id'=>'bizuno_api'];
    $settings[] = [
        'name'    => __('Bizuno Login Username', 'bizuno_api'),
        'desc_tip'=> __('This is the bizuno user login e-mail. It is best to make a new user that only has access to the Bizuno API and Contacts modules.', 'bizuno_api'),
        'id'      => 'bizuno_api_user',
        'type'    => 'text',
        //'css'   => 'min-width:300px;',
        'desc'    => __('We suggest making a new user in Bizuno for downloading orders. This is to track downloading orders to your system.', 'bizuno_api')];
    $settings[] = [
        'name'    => __('Bizuno Login Password', 'bizuno_api'),
        'id'      => 'bizuno_api_pw',
        'type'    => 'password',
        'desc'    => __('We suggest making a strong password.', 'bizuno_api')];
    $settings[] = [
        'name'    => __('Bizuno Download Url', 'bizuno_api'),
        'desc_tip'=> __('Leave blank if you Bizuno application is running on this site locally.', 'bizuno_api'),
        'id'      => 'bizuno_api_url',
        'type'    => 'text',
        'desc'    => __('If your Bizuno application is local, leave this field blank. For PhreeSoft hosted memberships, enter https://www.bizuno.com. For PhreeBooks 5 make sure the path to your PhreBooks homepage is entered, e.g. https://www.yoursite.com/PathToPhreeBooksHome.', 'bizuno_api')];
    $settings[] = [
        'name'    => __('Bizuno Order Prefix', 'bizuno_api'),
        'desc_tip'=> __('Leave blank for no prefix.', 'bizuno_api'),
        'id'      => 'bizuno_api_prefix',
        'type'    => 'text',
        'desc'    => __('Placing a value here will help identify where the orders originated from.', 'bizuno_api')];
    $settings[] = [
        'name'    => __('Bizuno Customer Prefix', 'bizuno_api'),
        'desc_tip'=> __('Leave blank for no prefix.', 'bizuno_api'),
        'id'      => 'bizuno_api_prefix_cust',
        'type'    => 'text',
        'desc'    => __('Placing a value here will help identify where your customers originated from.', 'bizuno_api')];
    $settings[] = [
        'name'    => __('Ship Complete Status', 'bizuno_api' ),
        'desc_tip'=> __('Default WooCommerce value is wc-completed.', 'bizuno_api'),
        'id'      => 'bizuno_ship_status',
        'type'    => 'select',
        'desc'    => __('If not set, the default WooCommerce value will be used.', 'bizuno_api'),
        'default' => 'wc-completed',
        'options' => wc_get_order_statuses()];
    $settings[] = [
        'name'    => __('Automatically Download Orders', 'bizuno_api' ),
        'id'      => 'bizuno_api_autodownload',
        'type'    => 'checkbox',
        'desc'    => __('If checked, your orders will automatically be downloaded to Bizuno and status at the cart marked complete just after the customer completes the order.', 'bizuno_api')];
    $settings[] = ['type'=>'sectionend', 'id'=>'bizuno_api'];
    return $settings;
}
